@@ -1,6 +1,5 @@
 # coding: utf-8
 
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 import os
 from datetime import datetime
@@ -134,7 +133,7 @@ class TasksMaterialsBuilder(AbstractBuilder):
             db_read = get_database(db_file, admin=False)
             db_read.collection_names()  # throw error if auth failed
         except:
-            logger.warn("Warning: could not get read-only database; using write creds")
+            logger.warning("Warning: could not get read-only database; using write creds")
             db_read = get_database(db_file, admin=True)
         return cls(db_write[m], db_write[c], db_read[t], **kwargs)
 
@@ -153,8 +152,8 @@ class TasksMaterialsBuilder(AbstractBuilder):
 
         Args:
             taskdoc (dict): a JSON-like task document
-            ltol (float): StructureMatcher tuning parameter 
-            stol (float): StructureMatcher tuning parameter 
+            ltol (float): StructureMatcher tuning parameter
+            stol (float): StructureMatcher tuning parameter
             angle_tol (float): StructureMatcher tuning parameter
 
         Returns:
