@@ -1,6 +1,5 @@
 # coding: utf-8
 
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 """
 This module defines the ferroelectric workflow
@@ -125,7 +124,7 @@ def get_wf_ferroelectric(polar_structure, nonpolar_structure, vasp_cmd="vasp", d
 
     # Add FireTask that uses Polarization object to store spontaneous polarization information
     if add_analysis_task:
-        fw_analysis = Firework(PolarizationToDb(db_file=db_file, name="_polarization_post_processing"),
+        fw_analysis = Firework(PolarizationToDb(db_file=db_file),
                                parents=interpolation, name="_polarization_post_processing")
         wf.append(fw_analysis)
 
